/*====================================================================================
This source file is part of the TechTools Plugin Development Kit. 
Copyright (c) 2011 by TechTools  

DISCLAIMERS:
 - NO WARRANTIES
     This software is provided 'As Is', without any express or implied
     warranty of any kind, including but not limited to any warranties 
     of merchantability, noninfringement, or fitness of a particular 
     purpose. The Copyright holders do not warrant or assume responsibility
     for the accuracy or completeness of any information contained within
     this software. 
 - LIMITATION OF LIABILITY
     In no event shall the copyright holders be liable for any damages 
     (including, without limitation, lost profits, business interruption, 
     or lost information) rising out of use of or inability to use this 
     software, even if advised of the possibility of such damages.  In no
     event will the copyright holders be liable for loss of data or for 
     indirect, special, incidental, consequential (including lost profit),
     or other damages based in contract, tort or otherwise. The copyright
     holders shall have no liability with respect to the content of the 
     software or any part thereof, including but not limited to errors or 
     omissions contained therein, libel, infringements of rights of publicity,
     privacy, trademark rights, business interruption, personal injury, 
     loss of privacy, moral rights or the disclosure of confidential
     information.  

Use and redistribution of this software or of derived works, in source or
compiled form is permitted as long as the following restrictions are observed:

- The above copyright(s), disclaimers and these restrictions are retained in the
  source files and, if distributed in compiled form, must be duplicated in 
  documentation or other materials and provided with the distribution.
- The derived work is used exclusively as a plugin to the TechTools DigiView 
  software, to process data captured with TechTools hardware.
- The copyright holders' names may not be used to endorse or to promote any 
  product or derived works.

Any other use is prohibited without express, written permission from TechTools.

email: support@tech-tools.com,  sales@tech-tools.com 
web: www.tech-tools.com   Voice:972-272-9392  FAX: 972-494-5814
======================================================================================*/

#pragma once

#include <SDKDDKVer.h>
#include <stdio.h>
#include <tchar.h>
#include <io.h>        // for _setmode
#include <fcntl.h>     // for _O_BINARY constant
#include <string>
#include <vector>      // for string list container

using namespace std;

typedef _int64 int64;
typedef unsigned _int64 uint64;

//enum FIELDTYPES {STARTFRAME, STARTFIELD, ENDFRAME, ENDFIELD, CNTLHALT, CNTLNOSAVE, CNTLSAVE};
enum CNTLCODES {CNTLHALT, CNTLNOSAVE, CNTLSAVE, CNTLUNKNOWN};  // note: CNTLUNKNOWN is a sentinal

// We use an 8 byte packet for all app<->plug-in communications. Define a 
// union to allow easy access to data as Int64, hi/low int32s or as array of 8 bytes.
union Data64
{
	unsigned _int8 bytes[8];
	struct {_int32 lowint; _int32 highint; };
	_int64  i64;  
};


// public routines from the command parser (OUR CODE)
void StartFrame(int64 timestamp, Data64 data, unsigned char FormatID);
void StartField(int64 timestamp, Data64 data, unsigned char FormatID);
void EndFrame(int64 timestamp);
void EndField(int64 timestamp);
void SendControl(int64 timestamp,unsigned char CntlCode);

void FindChannelLimits(uint64 mask, uint64 &HighestBit,uint64 &LowestBit);
unsigned _int64 pack(uint64 dat, uint64 mask,uint64 HighBit, uint64 LowBit);

// public routines in the post-parser (YOUR CODE)
void OnLoad();
void GetStrList(int ID, vector<string> &strl);
void SetInitItem(unsigned char ID, unsigned char subID, int value);
void SetCfgItem(unsigned char ID, unsigned char subID, int value);
void StartOfData();
void Parse(int64 timestamp, Data64 rawdata);
void EndOfData();
void OnUnLoad();
